"""Generator Exercises."""


def join():
    """Join an iterable of objects into a new string."""


def strip_lines():
    """Strip newline characters from the end of each line."""


def all_together():
    """String together all items from the given iterables."""


def parse_ranges():
    """Return generator of numbers corresponding to ranges in a string"""


def first_prime_over():
    """Return the first prime number over a given number."""


def get_primes_over():
    """Return primes over a given number."""


def head():
    """Return first n items of a given iterable."""
